from typing import Type, List

import pytest
from pydantic import BaseModel, Field

from pymultirole_plugins.v1.schema import Document
from pymultirole_plugins.v1.segmenter import SegmenterBase, SegmenterParameters


def test_segmenter():
    with pytest.raises(TypeError) as err:
        parser = SegmenterBase()
        assert parser is None
    assert "Can't instantiate abstract class SegmenterBase with abstract methods segment" in str(err.value)


def test_default_options():
    options = SegmenterParameters()
    assert options is not None


class DummyParameters(SegmenterParameters):
    foo: str = Field("foo", description="Foo")
    bar: float = Field(0.123456789, description="Bar")


class Dummysegmenter(SegmenterBase):
    """Dummy segmenter.
    """

    def segment(self, documents: List[Document], parameters: SegmenterParameters) -> List[Document]:
        parameters: DummyParameters = parameters
        documents[0].sentences = [{'start': 0, 'end': 1}]
        documents[0].boundaries = {parameters.foo: [{'name': parameters.bar, 'start': 0, 'end': 1}]}
        return documents

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return DummyParameters


def test_dummy():
    segmenter = Dummysegmenter()
    options = DummyParameters()
    docs: List[Document] =\
        segmenter.segment([Document(text="This is a test document.", metadata=options.dict())], options)
    assert len(docs[0].sentences) == 1
    assert len(docs[0].boundaries) == 1



from typing import Type, List
import pytest
from pydantic import BaseModel, Field
from pymultirole_plugins.v1.annotator import AnnotatorBase, AnnotatorParameters
from pymultirole_plugins.v1.schema import Document, Category


def test_annotator():
    with pytest.raises(TypeError) as err:
        parser = AnnotatorBase()
        assert parser is None
    assert "Can't instantiate abstract class AnnotatorBase with abstract methods annotate" in str(err.value)


def test_default_options():
    options = AnnotatorParameters()
    assert options is not None


class DummyParameters(AnnotatorParameters):
    foo: str = Field("foo", description="Foo")
    bar: float = Field(0.123456789, description="Bar")


class Dummyannotator(AnnotatorBase):
    """Dummy annotator.
    """

    def annotate(self, documents: List[Document], parameters: AnnotatorParameters) \
            -> List[Document]:
        parameters: DummyParameters = parameters
        documents[0].categories = [Category(labelName=parameters.foo, score=parameters.bar)]
        return documents

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return DummyParameters


def test_dummy():
    annotator = Dummyannotator()
    options = DummyParameters()
    docs: List[Document] =\
        annotator.annotate([Document(text="This is a test document", metadata=options.dict())], options)
    assert len(docs[0].categories) == 1
    assert docs[0].categories[0].labelName == options.foo
    assert docs[0].categories[0].score == options.bar


def test_singleton():
    annotator1 = Dummyannotator()
    annotator2 = Dummyannotator()
    assert id(annotator2) == id(annotator1)



import abc
from typing import Type, List
from pydantic import BaseModel

from pymultirole_plugins.v1 import ABCSingleton
from pymultirole_plugins.v1.schema import Document


class SegmenterParameters(BaseModel):
    pass


class SegmenterBase(metaclass=ABCSingleton):
    """Base class for example plugin used in the tutorial.
    """

    def __init__(self):
        pass

    @abc.abstractmethod
    def segment(self, documents: List[Document], options: SegmenterParameters) \
            -> List[Document]:
        """Segment the input documents and return the modified documents.

        :param documents: A list of annotated documents.
        :param options: options of the parser.
        :returns: List[Document].
        """

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return SegmenterParameters
